@@ -97,6 +97,27 @@ tracker_network_type_get_type (void)
   return g_define_type_id__volatile;
 }
 
+GType
+tracker_crawl_flags_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+ 
+  if (g_once_init_enter (&g_define_type_id__volatile)) {
+    static const GFlagsValue values[] = {
+      { TRACKER_CRAWL_FLAG_NONE, "TRACKER_CRAWL_FLAG_NONE", "none" },
+      { TRACKER_CRAWL_FLAG_NO_STAT, "TRACKER_CRAWL_FLAG_NO_STAT", "no-stat" },
+      { TRACKER_CRAWL_FLAG_FOLLOW_SYMLINKS, "TRACKER_CRAWL_FLAG_FOLLOW_SYMLINKS", "follow-symlinks" },
+      { 0, NULL, NULL }
+    };
+    GType g_define_type_id = 
+       g_flags_register_static (g_intern_static_string ("TrackerCrawlFlags"), values);
+      
+    g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+  }
+    
+  return g_define_type_id__volatile;
+}
+
 
 
 /* Generated data ends here */
